@@ -1,19 +1,56 @@
+import datetime
 from typing import Any, Union
 
 
 class PullRequest:
-    def __init__(self, title_prefix: str, title_body: str, title_suffix: str):
+    def __init__(
+        self,
+        title_prefix: str,
+        title_body: str,
+        title_suffix: str,
+        custom_body="",
+        custom_title="",
+    ):
+        self.custom_body = custom_body
+        self.custom_title = custom_title
         self.title_prefix = title_prefix
         self.title_body = title_body
         self.title_suffix = title_suffix
 
         self.changed_packages: list[str] = []
 
+    def __eq__(self, other: object) -> bool:
+        """
+        Aids with checking equality during unit tests.
+
+        :param other: Other PullRequest
+        :return: bool
+        """
+        if self.custom_body != getattr(other, "custom_body"):
+            return False
+
+        if self.custom_title != getattr(other, "custom_title"):
+            return False
+
+        if self.title_prefix != getattr(other, "title_prefix"):
+            return False
+
+        if self.title_body != getattr(other, "title_body"):
+            return False
+
+        if self.title_suffix != getattr(other, "title_suffix"):
+            return False
+
+        return self.changed_packages == getattr(other, "changed_packages")
+
     def add_package(self, name: str):
         self.changed_packages.append(name)
 
     @property
     def body(self) -> str:
+        if self.custom_body != "":
+            return self.custom_body
+
         return f"""This update contains changes from the following packages:
 
 {self.render_package_list()}
@@ -29,6 +66,9 @@ _This pull request has been created by [rcmt](https://rcmt.readthedocs.io/)._
 
     @property
     def title(self) -> str:
+        if self.custom_title != "":
+            return self.custom_title
+
         return f"{self.title_prefix} {self.title_body} {self.title_suffix}"
 
 
@@ -110,6 +150,15 @@ class Repository:
         """
         raise NotImplementedError("class does not implement Repository.name()")
 
+    def pr_created_at(self, pr: Any) -> datetime.datetime:
+        """
+        Returns the date and time at which the pull request was created.
+
+        :param pr: The pull request identifier as returned by find_open_pull_request().
+        :return: datetime.datetime
+        """
+        raise NotImplementedError("class does not implement Repository.pr_created_at()")
+
     @property
     def project(self) -> str:
         """
